/* $Id: IGColor.h,v 1.1 2001/04/22 09:09:34 formella Exp $ */

/*
 *  Copyright (C) 2001, Dr. Arno Formella
 *                      Universidade de Vigo
 *                      Departamento de Informatica
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License
 *          http://www.gnu.ai.mit.edu/copyleft/gpl.html
 *  for more details.
 */

#ifndef IGCOLOR_H
#define IGCOLOR_H

#include <fstream.h>
#include <GL/glut.h>

class IGColor {
private:
  /// the red component in the RGB-model
  double red;
  /// the green component in the RGB-model
  double green;
  /// the blue component in the RGB-model
  double blue;

  /// clamps v to interval 0..1
  double Clamp(double v) const;
public:
  /// default constructor produces black,
  /// can be used to produce gray tone
  IGColor(double gray=0.0);
  /// constructs color with given RGB-values
  IGColor(double r, double g, double b);
  IGColor(const IGColor& C);
  ~IGColor();
  void SetColor(void) const;
  IGColor& operator=(const IGColor& C);
  /// returns red component as rvalue
  double  Red(void) const { return red; }
  /// returns red component as lvalue
  double& Red(void) { return red; }
  /// returns green component as rvalue
  double  Green(void) const { return green; }
  /// returns green component as lvalue
  double& Green(void) { return green; }
  /// returns blue component as rvalue
  double  Blue(void) const { return blue; }
  /// returns blue component as lvalue
  double& Blue(void) { return blue; }

  /// reads the color from the input stream
  /// NOTE: is not the inverse of the operator<<
  friend istream& operator>>(istream& in, IGColor& C);
  /// writes the color to the output stream
  /// NOTE: is not the inverse of the operator>>
  friend ostream& operator<<(ostream& out, const IGColor& C);
};

#endif
