/* $Id: IGPolygon.h,v 1.1 2001/04/22 09:09:34 formella Exp $ */

/*
 *  Copyright (C) 2001, Dr. Arno Formella
 *                      Universidade de Vigo
 *                      Departamento de Informatica
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License
 *          http://www.gnu.ai.mit.edu/copyleft/gpl.html
 *  for more details.
 */

#ifndef IGPOLYGON_H
#define IGPOLYGON_H

#include <fstream.h>

#include "IGPoint.h"

class IGPolygon {
private:
  /// the list of points specifying the corner points of the polygon
  list<IGPoint> corners;
  /// the center of the polygon
  d3_point      center;
  /// the normal of the polygon
  vector    normal;
  /// the flag indicates whether the polygon is simple and convex,
  /// hence it is ok for OpenGL
  bool      okForOpenGL;

public:
  IGPolygon();
  IGPolygon(const IGPolygon& P);
  ~IGPolygon();
  IGPolygon& operator=(const IGPolygon& P);
private:
  /// checks whether the polygon can be used in OpenGL:
  /// - computes the center of the polygon
  /// - computes the normal vector of the polygon
  /// - if the normal can be computed:
  ///   > checks whether the largest bidimensional projection
  ///     of the polygon into the planes of the coordinate system
  ///     defines a simple and convex polygon
  ///   > sets the flag okForOpenGL apropriately
  void CheckForOpenGL(void);
  /// computes the center of the polygon
  void ComputeCenter(void);
  /// computes the normal vector of the polygon
  /// returns false if it is not possible to compute the vector
  bool ComputeNormal(void);
public:
  /// draws the polygon using OpenGL
  void Draw(void) const;
private:
  /// returns whether the list of bidimensional points defines
  /// a simple and convex polygon
  bool IsSimpleAndConvex(const list<point>& L) const;
public:
  /// returns whether the polygon can be used in OpenGL
  bool   isOkForOpenGL(void) const { return okForOpenGL; }
  /// returns the normal vector of the polygon
  vector Normal(void) const { return normal; }
  /// reads a polygon from the input stream
  /// NOTE: is not the inverse of the operator<<
  friend istream& operator>>(istream& in, IGPolygon& P);
  /// writes a polygon to the output stream
  /// NOTE: is not the inverse of the operator>>
  friend ostream& operator<<(ostream& out, const IGPolygon& P);
};

#endif
